/**
 * Tecnología del ocio
 * Bloque 1, unidad didáctica 11
 * Práctica 2
 */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <cwiid.h>
#include <math.h>
#include <time.h>

cwiid_err_t error; //Función para errores

//Función que imprime patrones
//patron es el array con el patrón a imprimir
//lng es la longitud del patrón a imprimir
void imprime_patron(int patron[], int lng) {
	int i;
	printf("[ ");
	for (i = 0; i < lng; i++) {
		if (patron[i] == 0) {printf("TUM");}
		else if (patron[i] == 1) {printf("PAK");}
		printf(" ");
	}
	printf("]\n");
}

//Función para errores
void error(cwiid_wiimote_t *wiimote, const char *s, va_list ap) {
	if (wiimote != NULL) {printf("Error en el dispositivo %d: ", cwiid_get_id(wiimote));}
	vprintf(s, ap);
	printf("\n");
}

int main(int argc, char *argv[]) {

	cwiid_wiimote_t* wiimote; //Gestor del wiimote
	bdaddr_t bdaddr; //Dirección Bluetooth del wiimote
	char tmp[256]; //Temporal para mostrar la dirección Bluetooth
	struct timespec t; //Instante de obtención del mensaje
	int mesg_count; //Número de mensajes obtenidos
	union cwiid_mesg *mesg; //Array de mensajes obtenido
	int i; //Índice del bucle que recorre los mensajes y elementos de patrones
	int salir = 0; //Control del bucle principal
	struct acc_cal calibracion_wm, calibracion_nc; //Datos de calibración de Wiimote y Nunchuk
	double accz_wm, accz_nc; //Aceleraciones en eje Z de Wiimote y Nunchuk
	double accz_old_wm, accz_old_nc; //Aceleraciones anteriores en eje Z de Wiimote y Nunchuk
	int patron[16], patron_usr[16]; //Patrón a reproducir y patrón reproducido por el usuario
	int lng_patron = 0, lng_patron_usr= 0; //Longitud útil del patrón a reproducir y longitud útil del patrón reproducido por el usuario

	cwiid_set_err(error); //Asigno la función de error

	printf("Poner el wiimote en modo de descubrimiento (pulsar 1+2) y pulsar enter...\n");
	getchar();
	printf("Buscando...\n");
	bdaddr = *BDADDR_ANY; //Se aceptará la conexión de cualquier wiimote
	if (!(wiimote = cwiid_connect(&bdaddr, CWIID_FLAG_MESG_IFC))) { //Conexión y habilitación de envío de mensajes
		printf("No se pudo detectar Wiimote\n");
	}
	else {
		//Conexión establecida
		ba2str(&bdaddr, tmp); //Traducción a cadena de caracteres de la dirección Bluetooth
		printf("Dirección Bluetooth del Wiimote %i: %s\n", cwiid_get_id(wiimote), tmp);
		//Comprobar la presencia de Nunchuk
		if (cwiid_set_rpt_mode(wiimote, CWIID_RPT_STATUS)) {
			printf("Error al establecer modo de informe\n");
		}
		else if (cwiid_get_mesg(wiimote, &mesg_count, &mesg, &t) != 0) {
			printf("Error al obtener mensajes del Wiimote\n");
		}
		else {
			for (i = 0; i < mesg_count; i++) {
				if (mesg[i].type == CWIID_MESG_STATUS) {
					if (!(mesg[i].status_mesg.ext_type & CWIID_EXT_NUNCHUK)) {
						printf("No se ha detectado Nunchuk, pulsa enter para salir.\n");
						salir = 1;
					}
				}
			}
		}
		//Si hay Nunchuk seguir
		if (!salir) {
			srand(time(NULL)); //Inicializar la semilla aleatoria
			if (cwiid_set_rpt_mode(wiimote, CWIID_RPT_NUNCHUK|CWIID_RPT_BTN|CWIID_RPT_ACC)) {
				printf("Error al establecer modo de informe\n");
			}
			else if (cwiid_get_acc_cal(wiimote, CWIID_EXT_NONE, &calibracion_wm)) {
				printf("No se pudieron obtener los datos de calibración del Wiimote\n");
			}
			else if (cwiid_get_acc_cal(wiimote, CWIID_EXT_NUNCHUK, &calibracion_nc)) {
				printf("No se pudieron obtener los datos de calibración del Nunchuk\n");
			}
			else {
				printf("Pulsa el botón Home para salir\n");
				while (!salir) {
					if (lng_patron == 16) { //Fin de juego
						printf("Has completado el juego, ¡muy bien!\n");
						salir = 1;
					}
					else if (lng_patron == lng_patron_usr) { //El usuario ha ejecutado los sonidos que debía
						if (lng_patron_usr > 0) { //Compruebo el patrón ejecutado
							i = 0;
							while ((i < lng_patron_usr) && (!salir)) {
								if (patron_usr[i] != patron[i]) {
									printf("Debías reproducir: ");
									imprime_patron(patron, lng_patron);
									printf("Has reproducido  : ");
									imprime_patron(patron_usr, lng_patron_usr);
									printf("Has fallado, fin del juego.\n");
									salir = 1;
								}
								i++;
							}
						}
						if (!salir) {
							patron[lng_patron] = rand() % 2;
							lng_patron++;
							printf("Patrón a reproducir: \n");
							imprime_patron(patron, lng_patron);
							lng_patron_usr = 0;
						}
					}
					if (!salir) {
						if (cwiid_get_mesg(wiimote, &mesg_count, &mesg, &t) != 0) {
							printf("Error al obtener mensajes del wiimote\n");
						}
						else {
							for (i = 0; i < mesg_count; i++) {
								if (mesg[i].type == CWIID_MESG_NUNCHUK) {
									accz_nc = (double)(mesg[i].nunchuk_mesg.acc[CWIID_Z] - calibracion_nc.zero[CWIID_Z])/(double)(calibracion_nc.one[CWIID_Z] - calibracion_nc.zero[CWIID_Z]);
									if (((accz_nc/fabs(accz_nc)) != (accz_old_nc/fabs(accz_old_nc))) && (accz_nc > 0) && (fabs(accz_nc-accz_old_nc) > 1)) {
										patron_usr[lng_patron_usr] = 1;
										lng_patron_usr++;
										printf("PAK\n");
									}
									accz_old_nc = accz_nc;
								}
								else if (mesg[i].type == CWIID_MESG_ACC) {
									accz_wm = (double)(mesg[i].acc_mesg.acc[CWIID_Z] - calibracion_wm.zero[CWIID_Z])/(double)(calibracion_wm.one[CWIID_Z] - calibracion_wm.zero[CWIID_Z]);
									if (((accz_wm/fabs(accz_wm)) != (accz_old_wm/fabs(accz_old_wm)))  && (accz_wm > 0) && (fabs(accz_wm-accz_old_wm) > 1)) {
										patron_usr[lng_patron_usr] = 0;
										lng_patron_usr++;
										printf("TUM\n");
									}
									accz_old_wm = accz_wm;
								}
								else if (mesg[i].type == CWIID_MESG_BTN) {
									if (mesg[i].btn_mesg.buttons = 128) {salir = 1;}
								}
								else {
									printf("Tipo de mensaje no esperado\n");
								}
							}
						}
					}
				}
			}
		}
		if (cwiid_disconnect(wiimote)) { //Desconexión del wiimote
			printf("Error al desconectar el wiimote\n");
		}
	}
	return 0;
}
